/*

*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
   This class builds the GUI, instantiates
   all the program objects, and makes sure
   that every object has a reference to any
   other object that it must communicate with.

   This class instantiates the JFrame that
   is the source of the key events.
*/
public class CircleProgram
{
   /**
      This constructor instantiates the GUI.
   */
   public CircleProgram()
   {
      final JFrame jf = new JFrame("Circle Program");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      final CircleState cs = new CircleState();     // Model
      final CirclePanel cp = new CirclePanel(cs);   // View
      jf.getContentPane().add(cp);
      jf.pack();
      jf.setLocationRelativeTo(null); // place the JFrame at the center of the desktop
      jf.requestFocusInWindow();
      jf.setVisible(true);

      CircleListener listener = new CircleListener(cs, cp); // Controller
      jf.addKeyListener(listener);   // delegation
      jf.addMouseListener(listener); // delegation
    //cp.addMouseListener(listener);  // This line fixes a bug.
    //cp.addKeyListener(listener);    // This line causes a bug.
   }//constructor

   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new CircleProgram()
      );
      System.out.println("bye"); // Notice that main() exits.
   }//main()
}

/*
Here is a picture showing some relevant objects in the JVM's heap.

    JFrame object
+----------------------+    List<KeyListener>     CircleListener object (Controller)
|  List<KeyListener>---|------->+------+        +----------------------------+
|                      |        |      |        | CircleState cs             |
|  addKeyListener()    |        +------+        | CirclePanel cp             |
+----------------------+        |  ----|------->|                            |<--------------------+
GUI object that receives        +------+        | keyTyped(KeyEvent e)       |                     |
KeyEvent objects from           |      |        | mouseClicked(MouseEvent e) |                     |
the JVM.                        +------+        +----------------------------+                     |
                                                 An object that implements                         |
                                                 the KeyListener interface and                     |
    KeyEvent object                              the MouseListener interface.                      |
  +-----------------+                                                                              |
  |  information    |                                                                              |
  |  from the OS    |      CircleState object        CirclePanel object (View)                     |
  |  about the      |      (Model)                  +---------------------+                        |
  |  keyboard event |      +-----------------+      | CircleState cs      |                        |
  +-----------------+      |  int xNew       |      |                     |    List<MouseListener> |
                           |  int yNew       |      | List<MouseListener>-|----->+------+          |
    MouseEvent object      |  int xOld       |      |                     |      |      |          |
  +-----------------+      |  int yOld       |      | addMouseListener()  |      +------+          |
  |  information    |      +-----------------+      |                     |      |  ----|----------+
  |  from the OS    |                               | paintComponent()    |      +------+
  |  about the      |                               +---------------------+      |      |
  |  mouse event    |                               GUI object that needs        +------+
  +-----------------+                               to be repainted after
                                                    an event occurs.
 */
