/*

*/

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

/**
   This program is roughly an implementation of the
   Model-view-controller (MVC) design pattern.

   In this version, the Model, View, and Controller classes
   are all defined as classes external to the program class.

   The Controller object holds references to the Model and View
   objects. The View object holds a reference the the Model object.
*/
public class EventExperiment_ver1
{
   public static void main(String[] args)
   {
      final JFrame f = new JFrame();
      f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      f.setTitle("Interactive GUI v1");
      f.setSize(600, 600);

      f.setLayout(new FlowLayout());

      f.setLocationRelativeTo(null);
      f.setVisible(true);

      // 1. Object that stores GUI state information.
      final Model model = new Model();

      // 2. GUI component to update when there is an event.
      final JPanel view = new View(model);
      view.setPreferredSize(new Dimension(100, 100));
      f.add(view);

      // 3. GUI component that is a source of events.
      final JButton b1 = new JButton("Change Color");
      f.add(b1);

      // 4. Object that handles events from the button.
      final ActionListener controller = new ButtonListener(model, view);

      // Register the listener with the button.
      b1.addActionListener(controller);
   }
}


// The Controller in MVC (defined as an external class).
class ButtonListener implements ActionListener
{
   public final Model state;
   public final JPanel jp;

   ButtonListener(final Model state, final JPanel jp){
      this.state = state;
      this.jp = jp;
   }
   @Override
   public void actionPerformed(ActionEvent e){
      state.incColorNumber();  // update the model
      jp.repaint();            // notify the view
   }
}


// The View in MVC.
@SuppressWarnings("serial")
class View extends JPanel
{
   public final Model state;

   View(final Model state){
      this.state = state;
   }
   @Override
   protected void paintComponent(Graphics g)
   {
      //System.out.println( state.getColorNumber() );
      final Color c = state.getColor();  // query the model
      this.setBackground(c);
      super.paintComponent(g);
   }
}


// The Model in MVC.
class Model
{
   private int colorNumber = 0;
   public void incColorNumber(){colorNumber = (colorNumber+1)%4;}
   public int getColorNumber(){return colorNumber;}
   public Color getColor(){
      if (0 == colorNumber)
         return Color.red;
      else if (1 == colorNumber)
         return Color.green;
      else if (2 == colorNumber)
         return Color.blue;
      else
         return Color.yellow;
   }
}

/*
Here is a picture showing some relevant objects in the JVM's heap.

   JButton object
+-----------------------+
|                       |  List<ActionListener>    ButtonListener object (the Controller)
| List<ActionListener>--|------>+------+         +------------------------------+
|                       |       |      |         | Model model------------------|-------+
|                       |       +------+         | JPanel jp--------------------|---+   |
| addActionListener()   |       |  ----|-------->|                              |   |   |
+-----------------------+       +------+         |                              |   |   |
GUI object that receives        |      |         |actionPerformed(ActionEvent e)|   |   |
ActionEvent objects from        +------+         |                              |   |   |
the JVM.                                         |                              |   |   |
                                                 +------------------------------+   |   |
                                                 An object that implements          |   |
   ActionEvent object                            the ActionListener interface.      |   |
  +-----------------+                                                               |   |
  |  Information    |                            +--------------------------------------+
  |  from the OS    |                            |                                  |
  |  about the      |        Model object        |    JPanel object (the View)      |
  |  button event.  |      +-----------------+   |   +------------------+           |
  +-----------------+      | int colorNumber |   |   | Model model--+   |           |
                           | getColor()      |<--+   |              |   |<----------+
                           |                 |<------|--------------+   |
                           |                 |       | paintComponent() |
                           +-----------------+       | repaint()        |
                                                     +------------------+
                                                     GUI object that needs
                                                     to be repainted after
                                                     an event occurs.
*/
