/*

*/

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

/**
   This program is roughly an implementation of the
   Model-view-controller (MVC) design pattern.

   In this version, the Controller class is defined as
   a local inner class in the program class. The event
   handler object has access to the program object's
   constructor's (final) local variables.

   https://docs.oracle.com/javase/tutorial/java/javaOO/localclasses.html
*/
public class EventExperiment_ver4
{
   public EventExperiment_ver4()
   {
      final JFrame f = new JFrame();
      f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      f.setTitle("Interactive GUI v4");
      f.setSize(600, 600);

      f.setLayout(new FlowLayout());

      f.setLocationRelativeTo(null);
      f.setVisible(true);

      // 1. Object that stores GUI state information.
      final Model model = new Model();

      // 2. GUI component to update when there is an event.
      final View view = new View(model);
      view.setPreferredSize(new Dimension(100, 100));
      f.add(view);

      // 3. GUI component that is a source of events.
      final JButton b1 = new JButton("Change Color");
      f.add(b1);

      // 4. Object that handles events from the button.
      //    The Controller in MVC, defined as a local class.
      class ButtonListener implements ActionListener
      {
         @Override
         public void actionPerformed(ActionEvent e){
            model.incColorNumber();  // update the model
            view.repaint();          // notify the view
         }
      }
      // Controler object.
      final ActionListener controller = new ButtonListener();

      // Register the listener with the button.
      b1.addActionListener(controller);
   }


   public static void main(String[] args)
   {
      new EventExperiment_ver4();
   }
}


// The View in MVC.
@SuppressWarnings("serial")
class View extends JPanel
{
   public final Model state;
   View(final Model state){
      this.state = state;
   }
   @Override
   protected void paintComponent(Graphics g)
   {
      //System.out.println( state.getColorNumber() );
      final Color c = state.getColor();  // query the model
      this.setBackground(c);
      super.paintComponent(g);
   }
}


// The Model in MVC.
class Model
{
   private int colorNumber = 0;
   public void incColorNumber(){colorNumber = (colorNumber+1)%4;}
   public int getColorNumber(){return colorNumber;}
   public Color getColor(){
      if (0 == colorNumber)
         return Color.red;
      else if (1 == colorNumber)
         return Color.green;
      else if (2 == colorNumber)
         return Color.blue;
      else
         return Color.yellow;
   }
}

/*
Here is a picture showing some relevant objects in the JVM's heap.

                                                EventExperiment_ver4 constructor
                                                activation record of final local
                                                variables.
                                                +------------------------+
                                                | final Model model------|--------------+
                                                | final JPanel view------|----------+   |
                                                |                        |          |   |
   JButton object                               |                        |          |   |
+----------------------+                        +----------------/\------+          |   |
|                      |  List<ActionListener>                   ||                 |   |
| List<ActionListener>-|------->+------+         ButtonListener  || nesting link    |   |
|                      |        |      |         object          ||  (closure)      |   |
|                      |        +------+        +----------------||------------+    |   |
| addActionListener()  |        |  ----|------->|                              |    |   |
+----------------------+        +------+        |                              |    |   |
GUI object that receives        |      |        |actionPerformed(ActionEvent e)|    |   |
ActionEvent objects from        +------+        |                              |    |   |
the JVM.                                        |                              |    |   |
                                                +------------------------------+    |   |
                                                An object that implements           |   |
   ActionEvent object                           the ActionListener interface.       |   |
  +-----------------+                                                               |   |
  |  Information    |                            +--------------------------------------+
  |  from the OS    |        Model object        |    JPanel object (the View)      |
  |  about the      |      +-----------------+   |   +------------------+           |
  |  button event.  |      | int colorNumber |<--+   | Model model--+   |           |
  +-----------------+      | getColor()      |       |              |   |<----------+
                           |                 |<------|--------------+   |
                           |                 |       | paintComponent() |
                           +-----------------+       | repaint()        |
                                                     +------------------+
                                                     GUI object that needs
                                                     to be repainted after
                                                     an event occurs.
*/
