/*

*/

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

/**
   This program is roughly an implementation of the
   Model-view-controller (MVC) design pattern.

   In this version, both the Controller and View classes
   are defined as anonymous local inner classes in the
   program class.

   https://docs.oracle.com/javase/tutorial/java/javaOO/anonymousclasses.html
*/
public class EventExperiment_ver6
{
   public EventExperiment_ver6()
   {
      final JFrame f = new JFrame();
      f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      f.setTitle("Interactive GUI v6");
      f.setSize(600, 600);

      f.setLayout(new FlowLayout());

      f.setLocationRelativeTo(null);
      f.setVisible(true);

      // 1. Object that stores GUI state information.
      final Model model = new Model();

      // 2. GUI component to update when there is an event,
      //    defined as an instance of an anonymous inner class.
      @SuppressWarnings("serial")
      final JPanel view = new JPanel(){
         @Override
         protected void paintComponent(Graphics g)
         {
            //System.out.println( state.getColorNumber() );
            final Color c = model.getColor();  // query the model
            this.setBackground(c);
            super.paintComponent(g);
      }};
      view.setPreferredSize(new Dimension(100, 100));
      f.add(view);

      // 4. GUI component that is a source of events.
      final JButton b1 = new JButton("Change Color");
      f.add(b1);

      // 3. Object that handles events from the button (the controller),
      //    defined as an instance of an anonymous inner class.
      b1.addActionListener(
         new ActionListener(){
            @Override
            public void actionPerformed(ActionEvent e){
               model.incColorNumber();  // update the model
               view.repaint();          // notify the view
      }});
   }


   public static void main(String[] args)
   {
      new EventExperiment_ver6();
   }
}


// The Model in MVC.
class Model
{
   private int colorNumber = 0;
   public void incColorNumber(){colorNumber = (colorNumber+1)%4;}
   public int getColorNumber(){return colorNumber;}
   public Color getColor(){
      if (0 == colorNumber)
         return Color.red;
      else if (1 == colorNumber)
         return Color.green;
      else if (2 == colorNumber)
         return Color.blue;
      else
         return Color.yellow;
   }
}

/*
Here is a picture showing some relevant objects in the JVM's heap.

                                                EventExperiment_ver6 constructor
                                                activation record of final local
                                                variables.
                                                +------------------------+
                                                | final Model model------|----------+
                                                | final JPanel view--+   |          |
                                                |                    |   |          |
                                                |             +------+   |          |
                                                +--/\---------|---/\-----+          |
                                                   ||         |   ||                |
                                   anonymous class || nesting |   ||                |
                                   object          || link    |   ||                |
                                   +---------------||-+       |   ||                |
                                   |                  |       |   ||                |
                                   | paintComponent() |<------+   ||                |
                                   | repaint()        |           ||                |
                                   +------------------+           ||                |
                                   GUI object that needs          ||                |
                                   to be repainted after          ||                |
                                   an event occurs.               ||                |
   JButton object                                                 ||                |
+----------------------+                                          ||                |
|                      |  List<ActionListener>                    ||                |
| List<ActionListener>-|------->+------+         anonymous class  || nesting link   |
|                      |        |      |         object           ||  (closure)     |
|                      |        +------+        +-----------------||-----------+    |
| addActionListener()  |        |  ----|------->|                              |    |
+----------------------+        +------+        |                              |    |
GUI object that receives        |      |        |actionPerformed(ActionEvent e)|    |
ActionEvent objects from        +------+        |                              |    |
the JVM.                                        |                              |    |
                                                +------------------------------+    |
                                                An object that implements           |
   ActionEvent object                           the ActionListener interface.       |
  +-----------------+                                                               |
  |  Information    |                                                               |
  |  from the OS    |        Model object                                           |
  |  about the      |      +-----------------+                                      |
  |  button event.  |      | int colorNumber |                                      |
  +-----------------+      | getColor()      |<-------------------------------------+
                           |                 |
                           |                 |
                           +-----------------+
*/
