/*

*/

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

/**
   This program is roughly an implementation of the
   Model-view-controller (MVC) design pattern BUT this
   version combines everything into one single object!
*/
@SuppressWarnings("serial")
public class EventExperiment_ver8 extends JPanel implements ActionListener
{
   private int colorNumber = 0;  // part of the Model

   /**
      this object is:
      a) the object that stores GUI state information (Model),
      b) a GUI component to update when there is an event (View),
      c) the object that handles events from the button (Controller).
   */
   public EventExperiment_ver8()
   {
      final JFrame f = new JFrame();
      f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      f.setTitle("Interactive GUI v8");
      f.setSize(600, 600);

      f.setLayout(new FlowLayout());

      f.setLocationRelativeTo(null);
      f.setVisible(true);

      this.setPreferredSize(new Dimension(100, 100));
      f.add(this); // this object is the view object

      // GUI component that is a source of events (the Controller).
      final JButton b1 = new JButton("Change Color");
      f.add(b1);

      // Register the listener with the button.
      b1.addActionListener(this); // this object is the listener object
   }

   // this object is an event listener for the JButton (the Controller).
   @Override
   public void actionPerformed(ActionEvent e){
      this.incColorNumber();  // update the model
      this.repaint();         // notify the view
   }

   // this object is a JPanel (the View).
   @Override
   protected void paintComponent(Graphics g)
   {
      final Color c = this.getColor(); // query the model
      this.setBackground(c);
      super.paintComponent(g);
   }

   // this object is our Model.
   private void incColorNumber(){
      colorNumber = (colorNumber+1)%4;
   }
   private Color getColor(){
      if (0 == colorNumber)
         return Color.red;
      else if (1 == colorNumber)
         return Color.green;
      else if (2 == colorNumber)
         return Color.blue;
      else
         return Color.yellow;
   }


   public static void main(String[] args)
   {
      new EventExperiment_ver8();
   }
}

/*
Here is a picture showing some relevant objects in the JVM's heap.

   JButton object                                  EventExperiment_ver8 object
+-----------------------+                         +-------------------------------+
|                       |  List<ActionListener>   |                               |
| List<ActionListener>--|------>+------+          | int colorNumber // model      |
|                       |       |      |          | getColor()                    |
|                       |       +------+          |                               |
| addActionListener()   |       |  ----|--------->| paintComponent() // view      |
+-----------------------+       +------+          | repaint()                     |
GUI object that receives        |      |          |                               |
ActionEvent objects from        +------+          | // controller                 |
the JVM.                                          | actionPerformed(ActionEvent e)|
                                                  |                               |
                                                  +-------------------------------+
   ActionEvent object                             An object that implements the
  +-----------------+                             ActionListener interface, and is
  |  Information    |                             a GUI object that needs to be
  |  from the OS    |                             repainted after an event occurs,
  |  about the      |                             and holds the GUI's state info.
  |  button event.  |
  +-----------------+

*/
